local SM = AceLibrary("SharedMedia-1.0")
local dewdrop = AceLibrary("Dewdrop-2.0")
local Table={}
local LogDisplay={}
Relog.Table=Table
Relog.LogDisplay=LogDisplay

local Columns={}
local CurTable
local CurColID=0
local MinWidth=40
local ColumnNames={}
local ColumnTags={}
local SortColumn
local SortDir=true
--How big is the lookup cache?
local MaxCache=100

function Columns:SetupDivider(Divider)
	Divider:SetMovable(true)
	Divider:EnableMouse(true)
	
	Divider:SetScript("OnEnter", function () SetCursor("interface\\addons\\Relog\\textures\\resize.blp") end)
	Divider:SetScript("OnLeave", function () ResetCursor() end)
	Divider:SetScript("OnMouseDown", function() 
						if ( ( ( not this.isLocked ) or ( this.isLocked == 0 ) ) and ( arg1 == "LeftButton" ) ) then
						  this:StartMoving();
						  this.isMoving = true;
						 end
						end)
		
	Divider:SetScript("OnMouseUp", function() 
						if ( this.isMoving ) then
							this:StopMovingOrSizing();
							local LeftDiff=this:GetLeft()-this:GetParent():GetLeft()
							this:ClearAllPoints()
							this:SetPoint("CENTER",this:GetParent(),"LEFT",LeftDiff+3.5,0)							
							this.isMoving = false;

							if this:GetLeft()<(this.Left:GetLeft()+MinWidth) then
								LeftDiff=this.Left:GetLeft()-this:GetParent():GetLeft()+MinWidth
								this:ClearAllPoints()
								this:SetPoint("CENTER",this:GetParent(),"LEFT",LeftDiff+3.5,0)	
							end

							if this:GetRight()>(this.Right:GetRight()-MinWidth) then
								LeftDiff=this.Right:GetRight()-this:GetParent():GetLeft()-MinWidth
								this:ClearAllPoints()
								this:SetPoint("CENTER",this:GetParent(),"LEFT",LeftDiff+3.5,0)	
							end

							if this.Left then
								this.Left:FixText()
							end

							if this.Right then
								this.Right:FixText()
							end
						end
						end)
	Divider:SetScript("OnHide", function() 
						if ( this.isMoving ) then
							this:StopMovingOrSizing();
							local LeftDiff=this:GetLeft()-this:GetParent():GetLeft()
							this:ClearAllPoints()
							this:SetPoint("CENTER",this:GetParent(),"LEFT",LeftDiff+3.5,0)
							this.isMoving = false;

							if this:GetLeft()<(this.Left:GetLeft()+MinWidth) then
								LeftDiff=this.Left:GetLeft()-this:GetParent():GetLeft()+MinWidth
								this:ClearAllPoints()
								this:SetPoint("CENTER",this:GetParent(),"LEFT",LeftDiff+3.5,0)	
							end

							if this:GetRight()>(this.Right:GetRight()-MinWidth) then
								LeftDiff=this.Right:GetRight()-this:GetParent():GetLeft()-MinWidth
								this:ClearAllPoints()
								this:SetPoint("CENTER",this:GetParent(),"LEFT",LeftDiff+3.5,0)	
							end

							if this.Left then
								this.Left:FixText()
							end

							if this.Right then
								this.Right:FixText()
							end
						 end
						end)
	Divider:SetScript("OnUpdate", function() 
						if ( this.isMoving ) then
							local LeftDiff=this:GetLeft()-this:GetParent():GetLeft()
							this:ClearAllPoints()
							this:SetPoint("CENTER",this:GetParent(),"LEFT",LeftDiff+3.5,0)

							if this:GetLeft()<(this.Left:GetLeft()+MinWidth) then
								LeftDiff=this.Left:GetLeft()-this:GetParent():GetLeft()+MinWidth
								this:ClearAllPoints()
								this:SetPoint("CENTER",this:GetParent(),"LEFT",LeftDiff+3.5,0)	
							end

							if this:GetRight()>(this.Right:GetRight()-MinWidth) then
								LeftDiff=this.Right:GetRight()-this:GetParent():GetLeft()-MinWidth
								this:ClearAllPoints()
								this:SetPoint("CENTER",this:GetParent(),"LEFT",LeftDiff+3.5,0)	
							end

							if this.Left then
								this.Left:FixText()
							end

							if this.Right then
								this.Right:FixText()
							end
						end
						end)	
end

function Columns.RemoveMe(id)
	CurTable:RemoveColumn(id)
	dewdrop:Close()
end

function Columns.ChangeMe(column)
	Relog:ShowEditColumn(column)
	dewdrop:Close()
end

function Columns.ShowDropdown()
	dewdrop:AddLine('text',"Remove Column",'notCheckable', true,'func',Columns.RemoveMe,'arg1',this.id)
	dewdrop:AddLine('text',"Edit Column",'notCheckable', true,'func',Columns.ChangeMe,'arg1',this:GetParent())
end

function Columns:SetRowNum(num)	
	if num>#self.Rows then
		for i=(#self.Rows+1),num do
			self.Rows[i]=self:CreateFontString(nil,"OVERLAY","GameFontNormal")
			self.Rows[i]:SetTextColor(1.0,1.0,1.0)			
		end
	elseif num<#self.Rows then
		for i=num+1,#self.Rows do
			self.Rows[i]:Hide()
		end
	end

	self.Title:SetHeight(Relog.db.profile.RowHeight)
	self.Title.Text:SetFont(SM:Fetch("font",Relog.db.profile.Font),0.95*Relog.db.profile.RowHeight)

	for i=1,num do
		self.Rows[i]:SetFont(SM:Fetch("font",Relog.db.profile.Font),0.95*Relog.db.profile.RowHeight)
		self.Rows[i]:SetPoint("TOP",self,"TOP",0,-i*(1+Relog.db.profile.RowHeight))
		self.Rows[i]:Show()
	end
end

function Columns:SetRow(Row,Table)
	local FoundCache=nil
	for k,v in pairs(self.CacheID) do
		if v==Table.Time then
			FoundCache=k
		end
	end

	if FoundCache then
		self.Rows[Row]:SetText(self.CacheValue[FoundCache])
	else
		local NewVal=Relog.Tags.CompileTag(self.Tag,Table)
		self.CacheID[self.NextCache]=Table.Time
		self.CacheValue[self.NextCache]=NewVal
		
		--Inc next cache location
		self.NextCache=self.NextCache+1
		if self.NextCache>MaxCache then
			self.NextCache=1
		end

		self.Rows[Row]:SetText(NewVal)
	end

	local MaxWidth
	if self:GetRight() and self:GetLeft() then
		MaxWidth=self:GetRight() - self:GetLeft()
	else
		MaxWidth=self:GetWidth() 
	end
	local Text=self.Rows[Row]:GetText()

	self.Rows[Row].Text=Text

	while Text and self.Rows[Row]:GetStringWidth()>MaxWidth do
		Text=strsub(Text,1,#Text-1)
		self.Rows[Row]:SetText(Text.."...")
		if Text=="" then
			break
		end
	end
end

function Columns:FixText()
	local Text
	local MaxWidth
	if self:GetRight() and self:GetLeft() then
		MaxWidth=self:GetRight() - self:GetLeft()
	else
		MaxWidth=self:GetWidth() 
	end
	for _,Row in pairs(self.Rows) do
		Text=Row.Text
		Row:SetText(Text)
		while Text and Row:GetStringWidth()>MaxWidth do
			Text=strsub(Text,1,#Text-1)
			Row:SetText(Text.."...")
			if Text=="" then
				break
			end
		end
	end
end

function Table:ResetColumns()
	for k,v in pairs(self.Columns) do
		v:Hide()
		table.insert(self.ExtraColumns,v)
		self.Columns[k]=nil
		for k in pairs(v.CacheID) do
			v.CacheID[k]=nil
		end
	end

	for k,v in pairs(self.Dividers) do
		v:Hide()
		table.insert(self.ExtraDividers,v)
		self.Dividers[k]=nil
	end
end

function Table:RemoveColumn(num)
	table.remove(ColumnNames,num)
	table.remove(ColumnTags,num)
	self:ResetColumns()

	self.LockColumns=true

	for k, v in pairs(ColumnNames) do
		self:AddColumn(v,ColumnTags[k],k)
	end

	self.LockColumns=false

	self:UpdateRows()
end


function Table:AddColumn(Name,Tag,id)
	local Column

	local CurID
	if not self.LockColumns then
		CurID=#ColumnNames+1
		ColumnNames[CurID]=Name
		ColumnTags[CurID]=Tag
	else
		CurID=id
	end

	
	if #self.ExtraColumns>0 then
		Column=self.ExtraColumns[#self.ExtraColumns]
		self.ExtraColumns[#self.ExtraColumns]=nil

		for k in pairs(Column.CacheID) do
			Column.CacheID[k]=nil
		end
		Column.NextCache=1
		Column:Show()
	else
		Column=CreateFrame("FRAME",nil,self)
		
		Column.Rows={}
		Column.SetRowNum=Columns.SetRowNum
		Column.SetRow=Columns.SetRow
		Column.FixText=Columns.FixText

		Column.Title=CreateFrame("FRAME",nil,Column)
		Column.Title:SetHeight(Relog.db.profile.RowHeight)
		Column.Title:SetPoint("TOPLEFT",Column,"TOPLEFT")
		Column.Title:SetPoint("TOPRIGHT",Column,"TOPRIGHT")
		Column.Title:EnableMouse(true)
		Column.Title:SetScript("OnMouseDown",function() if arg1=="RightButton" then 
									dewdrop:Open(this)
								elseif arg1=="LeftButton" then
									if SortColumn~=this:GetParent() then
										if SortColumn then
											SortColumn.Title.Sort:Hide()
										end
										SortColumn=this:GetParent()
										Relog:SortLog(SortColumn.Tag)
										SortDir=true

										this.Sort:Show()
										this.Sort:SetTexCoord(0,0.6,0,1)
									else
										if SortDir then
											Relog:SortReverse()
											this.Sort:SetTexCoord(0,0.6,1,0)
											SortDir=false
										else
											this.Sort:Hide()
											SortColumn=nil
										end
									end
									CurTable:UpdateRows()
								end end)
		dewdrop:Register(Column.Title,'children',Columns.ShowDropdown)


		Column.Title.Text=Column.Title:CreateFontString(nil,"OVERLAY","GameFontNormal")
		Column.Title.Text:SetTextColor(1.0,1.0,1.0)
		Column.Title.Text:SetFont(SM:Fetch("font",Relog.db.profile.Font),0.95*Relog.db.profile.RowHeight)
		Column.Title.Text:SetPoint("CENTER",Column.Title,"CENTER")

		Column.Title.Sort=Column.Title:CreateTexture(nil,"ARTWORK")
		Column.Title.Sort:SetTexture("Interface\\Buttons\\UI-SortArrow.blp")
		Column.Title.Sort:SetWidth(Relog.db.profile.RowHeight)
		Column.Title.Sort:SetHeight(0.8*Relog.db.profile.RowHeight)
		Column.Title.Sort:SetPoint("RIGHT",Column.Title,"RIGHT")
		Column.Title.Sort:Hide()

		--Cache implemented to save lookups, overly simple to keep from too large of memory usage
		Column.NextCache=1
		Column.CacheID={}
		Column.CacheValue={}

		

		Column.Title.Background=Column.Title:CreateTexture(nil,"BACKGROUND")
		Column.Title.Background:SetAllPoints(Column.Title)
	end
	
	Column.id=CurID
	Column.Title.id=CurID
	Column.Tag=Tag
	Column:SetWidth(100)
	Column:SetHeight(self:GetHeight()-2)
	Column.Title.Text:SetText(Name)
	if math.fmod(#self.Columns,2)==0 then
		Column.Title.Background:SetTexture(0,0,0,0.4)
	else
		Column.Title.Background:SetTexture(0.3,0.3,0.3,0.4)
	end

	if #self.Columns==0 then
		Column:SetPoint("TOPLEFT",self,"TOPLEFT",1,-1)
	else
		local Divider
		if #self.ExtraDividers>0 then
			Divider=self.ExtraDividers[#self.ExtraDividers]
			self.ExtraDividers[#self.ExtraDividers]=nil
			Divider:Show()
		else			
			Divider=CreateFrame("FRAME",nil,self)
			Divider:SetFrameLevel(Divider:GetFrameLevel()+1)

			Divider.Background=Divider:CreateTexture(nil,"OVERLAY")
			Divider.Background:SetWidth(1)
			Divider.Background:SetPoint("TOP",Divider,"TOP")
			Divider.Background:SetPoint("BOTTOM",Divider,"BOTTOM")
			Divider.Background:SetTexture(1.0,1.0,1.0)
		end
		Divider:SetWidth(7)
		Divider:SetHeight(self:GetHeight()-2)
		Divider:ClearAllPoints()
		Divider:SetPoint("CENTER",self,"LEFT",10,0)
		Columns:SetupDivider(Divider)

		self.Dividers[#self.Dividers+1]=Divider
		
		Divider.Left=self.Columns[#self.Columns]
		Divider.Right=Column
		self.Columns[#self.Columns]:SetPoint("BOTTOMRIGHT",Divider,"BOTTOM")
		Column:SetPoint("TOPLEFT",Divider,"TOP")
	end
	Column:SetPoint("BOTTOMRIGHT",self,"BOTTOMRIGHT",-1,1)

	self.Columns[#self.Columns+1]=Column
	self:UpdateDividers()
end

function Table:UpdateDividers()
	local DividerNum=#self.Dividers
	local Spacing=(self:GetWidth()-2)/(DividerNum+1)

	for k, v in pairs(self.Dividers) do
		v:SetPoint("CENTER",self,"LEFT",1+k*Spacing,0)
	end
end


function Table:SetWidth(width)
	--First need to get divider spacing
	local Left=self:GetLeft()
	local WidthScale=width/self:GetWidth()

	for k, v in pairs(self.Dividers) do
		if Left and v:GetCenter() then
			v:SetPoint("CENTER",self,"LEFT",(v:GetCenter()-Left)*WidthScale,0)
		end
	end

	if #self.Columns>0 then
		MinWidth=math.min(40,width/#self.Columns)
	end

	self:OldSetWidth(width)
end

function Table:SetHeight(height)
	if height==self:GetHeight() then
		return
	end
	for k, v in pairs(self.Dividers) do
		v:SetHeight(height-2)
	end
	self:OldSetHeight(height)
	
	self:UpdateRows()
end

local TypeColors={
	Damage={1.0,0.2,0.2},
	Heal={0.2,1.0,0.2},
	Gain={0.2,0.2,1.0},
	Fail={1.0,1.0,0.2},
}

function Table:AtBottom()
	local offset = FauxScrollFrame_GetOffset(CurTable.ScrollBar)
	local RowNum=math.floor(CurTable:GetHeight()/(1+Relog.db.profile.RowHeight)-1)

	local size=Relog:GetFilteredSize()

	if (offset+RowNum)>=size then
		return true
	end
	return false
end

function Table:SetPosBottom()
	local scrollbar = getglobal("Relog_Table_ScrollBarScrollBar")
	

	local RowNum=math.floor(CurTable:GetHeight()/(1+Relog.db.profile.RowHeight)-1)
	local size=Relog:GetFilteredSize()
	local offset=size-RowNum

	if size<RowNum then
		return
	end
	
	local min, max = scrollbar:GetMinMaxValues()

	scrollbar:SetValue(max*offset/(size-RowNum)+min)

	self.ScrollBar.offset = offset
	self:UpdateRows()
end

function Table:FixColorRows()
	for k, v in pairs(self.ColorRows) do
		v:SetHeight(Relog.db.profile.RowHeight+1)
		v:SetPoint("TOP",CurTable,"TOP",0,-k*(1+Relog.db.profile.RowHeight))
	end
end

function Table:UpdateRows()	
	local offset = FauxScrollFrame_GetOffset(CurTable.ScrollBar)
	local RowNum=math.floor(CurTable:GetHeight()/(1+Relog.db.profile.RowHeight)-1)

	local size=Relog:GetFilteredSize()
	FauxScrollFrame_Update(CurTable.ScrollBar, size, RowNum, Relog.db.profile.RowHeight)

	if Relog.TableWindow.Log then
		Relog.TableWindow.Log:UpdateRows()
	end

	if not CurTable:IsVisible() then
		return
	end
	
	if (offset+RowNum)>size then
		RowNum=math.max(size-offset,0)
	end

	local CurRowN
	local CurRow
	while #CurTable.ColorRows<RowNum do
		CurRowN=#CurTable.ColorRows+1
		CurRow=CurTable:CreateTexture(nil,"ARTWORK")
		CurRow:SetTexture(1,1,1,1)
		CurRow:SetWidth(CurTable:GetWidth()-2)
		CurRow:SetHeight(Relog.db.profile.RowHeight+1)
		CurRow:SetPoint("TOP",CurTable,"TOP",0,-CurRowN*(1+Relog.db.profile.RowHeight))
		CurTable.ColorRows[CurRowN]=CurRow
	end

	if #CurTable.ColorRows>RowNum then
		for i=RowNum+1,#CurTable.ColorRows do
			CurTable.ColorRows[i]:Hide()
		end
	end

	for k,v in pairs(CurTable.Columns) do
		v:SetRowNum(RowNum)
	end

	for i=1,RowNum do
		local log
		if SortColumn then
			log=Relog:GetSortedLog(i+offset)
		else
			log=Relog:GetFilteredLog(i+offset)
		end
		for k,v in pairs(CurTable.Columns) do
			v:SetRow(i,log)
		end
		if TypeColors[log.Type] then
			local Col=TypeColors[log.Type]
			CurTable.ColorRows[i]:SetVertexColor(Col[1],Col[2],Col[3],0.2)
			CurTable.ColorRows[i]:Show()
			CurTable.ColorRows[i]:SetWidth(CurTable:GetWidth()-2)
		else
			CurTable.ColorRows[i]:Hide()
		end
	end

	Relog:ScheduleEvent("FixColums",CurTable.FixColumns,0.05,CurTable)
	
end

function Table:FixColumns()
	for k,v in pairs(CurTable.Columns) do
		v:FixText()
	end
end

function Table:CreateTable(parent)
	local theFrame=CreateFrame("FRAME",nil,parent)

	theFrame:ClearAllPoints()
	theFrame:SetPoint("CENTER",UIParent)
	theFrame:SetHeight(172)
	theFrame:SetWidth(400)

	theFrame:SetBackdrop({
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
		edgeFile = "Interface\\Buttons\\WHITE8X8.blp", edgeSize = 1,
		insets = {left = 1, right = 1, top = 1, bottom = 1},
	})
	theFrame:SetBackdropBorderColor(1.0,1.0,1.0,1)
	theFrame:SetBackdropColor(24/255, 24/255, 24/255,0)
	
	theFrame.Columns={}
	theFrame.Dividers={}
	theFrame.ExtraColumns={}
	theFrame.ExtraDividers={}
	theFrame.AddColumn=Table.AddColumn
	theFrame.UpdateDividers=Table.UpdateDividers
	theFrame.UpdateRows=Table.UpdateRows
	theFrame.OldSetWidth=theFrame.SetWidth
	theFrame.OldSetHeight=theFrame.SetHeight
	theFrame.SetWidth=Table.SetWidth
	theFrame.SetHeight=Table.SetHeight
	theFrame.FixColumns=Table.FixColumns
	theFrame.ResetColumns=Table.ResetColumns
	theFrame.RemoveColumn=Table.RemoveColumn
	theFrame.AtBottom=Table.AtBottom
	theFrame.SetPosBottom=Table.SetPosBottom
	theFrame.FixColorRows=Table.FixColorRows

	theFrame.ColorRows={}
	

	theFrame.ScrollBar=CreateFrame("SCROLLFRAME","Relog_Table_ScrollBar",parent,"FauxScrollFrameTemplate")
	theFrame.ScrollBar:SetScript("OnVerticalScroll", function() FauxScrollFrame_OnVerticalScroll(12, theFrame.UpdateRows) end)
	theFrame.ScrollBar:SetPoint("TOPLEFT",theFrame,"TOPLEFT",0,-Relog.db.profile.RowHeight)	
	theFrame.ScrollBar:SetPoint("BOTTOMRIGHT",theFrame,"BOTTOMRIGHT",0,0)

	CurTable=theFrame
	return theFrame
end

function LogDisplay:UpdateRows()
	local offset = FauxScrollFrame_GetOffset(CurTable.ScrollBar)
	local RowNum=math.floor(self:GetHeight()/(1+Relog.db.profile.RowHeight))

	local size=Relog:GetFilteredSize()

	if not self:IsVisible() then
		return
	end
	
	if (offset+RowNum)>size then
		RowNum=math.max(size-offset,0)
	end

	if (#self.Rows)<RowNum then
		for i=#self.Rows+1,RowNum do
			self.Rows[i]=self:CreateFontString(nil,"OVERLAY","GameFontNormal")
			self.Rows[i]:SetTextColor(1.0,1.0,1.0)
			self.Rows[i]:SetFont(SM:Fetch("font",Relog.db.profile.Font),0.95*Relog.db.profile.RowHeight)
			self.Rows[i]:SetPoint("TOPLEFT",self,"TOPLEFT",1,-(i-1)*(1+Relog.db.profile.RowHeight))
		end
	end

	for i=RowNum+1,#self.Rows do
		self.Rows[i]:Hide()
	end

	for i=1,RowNum do
		local log=Relog:GetFilteredLog(i+offset)
		local Time=Relog.Tags.CompileTag("[Date(\"%H:%M:%S.%s\",Time)]",log)
		self.Rows[i]:SetText(Time.." "..log.CompiledCache)
		self.Rows[i]:Show()
	end
end

function LogDisplay:FixRows()
	for k, v in pairs(self.Rows) do
		self.Rows[k]:SetFont(SM:Fetch("font",Relog.db.profile.Font),0.95*Relog.db.profile.RowHeight)
		self.Rows[k]:SetPoint("TOPLEFT",self,"TOPLEFT",1,-(k-1)*(1+Relog.db.profile.RowHeight))
	end
end
function LogDisplay:CreateLog(parent)
	local theFrame=CreateFrame("FRAME",nil,parent)

	theFrame:ClearAllPoints()
	theFrame:SetPoint("CENTER",UIParent)
	theFrame:SetHeight(172)
	theFrame:SetWidth(400)

	theFrame.UpdateRows=LogDisplay.UpdateRows
	theFrame.FixRows=LogDisplay.FixRows
	theFrame.Rows={}
	return theFrame
end

function Relog:CreateTableWindow()
	local theFrame=Relog:CreateFrame("Relog_Table","Relog - Table Display",432,400)
	Relog.TableWindow=theFrame

	

	theFrame:SetResizable(true)
	theFrame:SetMinResize(140,63)
	theFrame:SetMaxResize(800,820)	
	theFrame:SetScript("OnSizeChanged", function() this.Table:SetWidth(this:GetWidth()-28);this.Table:SetHeight(this:GetHeight()-34);this.Log:SetWidth(this:GetWidth()-28);this.Log:SetHeight(this:GetHeight()-34) end)

	theFrame.DragBottomRight = CreateFrame("Frame", nil, theFrame)
	theFrame.DragBottomRight:Show()
	theFrame.DragBottomRight:SetFrameLevel( theFrame:GetFrameLevel() + 10)
	theFrame.DragBottomRight:SetWidth(16)
	theFrame.DragBottomRight:SetHeight(16)
	theFrame.DragBottomRight:SetPoint("BOTTOMRIGHT", theFrame, "BOTTOMRIGHT", 0, 0)
	theFrame.DragBottomRight:EnableMouse(true)
	theFrame.DragBottomRight:SetScript("OnMouseDown", function() this:GetParent().isResizing = true; this:GetParent():StartSizing("BOTTOMRIGHT") end )
	theFrame.DragBottomRight:SetScript("OnMouseUp", function() this:GetParent():StopMovingOrSizing(); this:GetParent().isResizing = false; end )


	theFrame.DragBottomLeft = CreateFrame("Frame", nil, theFrame)
	theFrame.DragBottomLeft:Show()
	theFrame.DragBottomLeft:SetFrameLevel( theFrame:GetFrameLevel() + 10)
	theFrame.DragBottomLeft:SetWidth(16)
	theFrame.DragBottomLeft:SetHeight(16)
	theFrame.DragBottomLeft:SetPoint("BOTTOMLEFT", theFrame, "BOTTOMLEFT", 0, 0)
	theFrame.DragBottomLeft:EnableMouse(true)
	theFrame.DragBottomLeft:SetScript("OnMouseDown", function() this:GetParent().isResizing = true; this:GetParent():StartSizing("BOTTOMLEFT") end )
	theFrame.DragBottomLeft:SetScript("OnMouseUp", function() this:GetParent():StopMovingOrSizing(); this:GetParent().isResizing = false; end )

	theFrame.AddColumn=CreateFrame("Button",nil,theFrame)
	theFrame.AddColumn:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-Up.blp")
	theFrame.AddColumn:SetPushedTexture("Interface\\Buttons\\UI-PlusButton-Down.blp")
	theFrame.AddColumn:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight.blp")
	theFrame.AddColumn:SetWidth(16)
	theFrame.AddColumn:SetHeight(16)
	theFrame.AddColumn:SetPoint("TOPRIGHT",theFrame,"TOPRIGHT",-22,-14)
	theFrame.AddColumn:SetScript("OnClick",function() Relog:ShowAddColumn() end)

	theFrame.Filter=CreateFrame("Button",nil,theFrame)
	theFrame.Filter:SetNormalTexture("Interface\\Icons\\INV_Misc_Net_01.blp")
	theFrame.Filter:SetPushedTexture("Interface\\Icons\\INV_Misc_Net_01.blp")
	theFrame.Filter:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight.blp")
	theFrame.Filter:SetWidth(16)
	theFrame.Filter:SetHeight(16)
	theFrame.Filter:SetPoint("RIGHT",theFrame.AddColumn,"LEFT",-2,0)
	theFrame.Filter:SetScript("OnClick",function() Relog:ShowFilter() end)

	theFrame.Switch=CreateFrame("Button",nil,theFrame)
	theFrame.Switch:SetNormalTexture("Interface\\Buttons\\UI-GuildButton-PublicNote-Up.blp")
	theFrame.Switch:SetPushedTexture("Interface\\Buttons\\UI-GuildButton-PublicNote-Down.blp")	
	theFrame.Switch:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight.blp")
	theFrame.Switch:SetWidth(16)
	theFrame.Switch:SetHeight(16)
	theFrame.Switch:SetPoint("RIGHT",theFrame.Filter,"LEFT",-2,0)
	theFrame.Switch:SetScript("OnClick",function() 
						local parent=this:GetParent()
						if parent.Table:IsShown() then
							parent.Table:Hide()
							parent.Log:Show()
							parent.Title:SetText("Relog - Log Display")
						else
							parent.Table:Show()
							parent.Log:Hide()
							parent.Title:SetText("Relog - Table Display")
						end
					 end)

	

	local a=Relog.Table:CreateTable(theFrame)
	a:AddColumn("Time","[Date(\"%H:%M:%S.%s\",Time)]")
	a:AddColumn("Type","[Type]")
	a:AddColumn("Source","[Source]")
	a:AddColumn("Victim","[Victim]")
	a:AddColumn("Amount","[Amount]")
	a:AddColumn("Ability","[Ability]")
	a:AddColumn("Result","[Result]")
	
	a:SetPoint("TOPLEFT",theFrame,"TOPLEFT",2,-32)
	a:SetParent(theFrame)
	a:SetWidth(400-32)
	a:SetHeight(398)
	
	theFrame.Table=a

	local LogDisplay=LogDisplay:CreateLog(theFrame)
	theFrame.Log=LogDisplay
	theFrame.Log:SetPoint("TOPLEFT",theFrame,"TOPLEFT",2,-32)
	theFrame.Log:Hide()

	Relog:FilterLog("(Source==Player)|(Victim==Player)")

	a:UpdateRows()	
	theFrame.Log:UpdateRows()

	Relog.WindowOrder:AddWindow(theFrame)
end